==> pvfs-1.3.2.tgz: pvfs-1.3.2/lib/pvfs_unlink.c <==
/* pvfs_unlink.c copyright (c) 1996 Clemson University, all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_unlink.c,v $
 * Revision 1.10  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.9  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.7  1998/07/02 18:36:01  rbross
 * modified to use new check_pvfs() return values
 *
 * Revision 1.6  1998/02/16 19:18:07  rbross
 * Newt patches
 *
 * Revision 1.5  1997/10/27 21:22:01  pvfs
 * added license
 *
 * Revision 1.4  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.3  1997/01/20 17:21:13  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1997/01/16 19:39:59  mcettei
 * uses debugging code
 *
/*/

static char pvfs_unlink_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_unlink.c,v 1.10 1998/07/10 20:20:00 rbross Exp $";

/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_unlink(const char* pathname)
{
	int i, s;
	mreq request;
	mack ack;
	dmeta dir;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	/* Check to see if file is a PVFS file */
	if ((i = check_pvfs(pathname)) != 1) {
		if (i == 2) /* PVFS directory */ {
			errno = EISDIR;
			return(-1);
		}
		/* otherwise let UNIX handle giving back an error */
		return (syscall(SYS_unlink, pathname));
	}
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_UNLINK;
	request.dsize = strlen(dir.rd_path);
	request.req.unlink.fname = 0;

	/* Send request to mgr */	
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}

==> pvfs-1.4.2.tgz: pvfs-1.4.2/lib/pvfs_unlink.c <==
/* pvfs_unlink.c copyright (c) 1996 Clemson University, all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

 */

static char pvfs_unlink_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_unlink.c,v 1.3 1999/10/15 22:05:57 rbross Exp $";

/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <string.h>
#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_unlink(const char* pathname)
{
	int i=0, s, len;
	mreq request;
	mack ack;
#ifdef __USE_NFS__
	dmeta dir;
#else
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;
#endif

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}
	
	/* Check to see if file is a PVFS file */
#ifdef __USE_NFS__
	if (pvfs_checks_disabled || (i = check_pvfs(pathname)) != 1) {
#else
	if (pvfs_checks_disabled
	|| (i = pvfs_detect(pathname, &fn, &saddr, &fs_ino)) != 1)
	{
#endif
		if (i == 2) /* PVFS directory */ {
			errno = EISDIR;
			return(-1);
		}
		/* check for .pvfsdir and .iodtab, reported back as UNIX files */
		len = strlen(pathname);
		if (!strncmp(".pvfsdir", &pathname[len-8], 8) 
		|| !strncmp(".iodtab", &pathname[len-7], 7))
		{
			return(0); /* pretend everything went as planned, but don't
							* actually remove.  This is to make recursive
							* removes and such work w/out error.
							*/
		}


		/* otherwise let UNIX handle giving back an error */
		return (syscall(SYS_unlink, pathname));
	}
#ifdef __USE_NFS__
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}
#endif

	/* Prepare request for file system  */
#ifdef __USE_NFS__
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.dsize = strlen(dir.rd_path);
#else
	request.dsize = strlen(fn);
#endif
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_UNLINK;
	request.req.unlink.fname = 0;

	/* Send request to mgr */	
#ifdef __USE_NFS__
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
#else
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
#endif
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}
